--
-- InteractiveButtons v3.0
--
-- Interface: 1.4.4.0 1.4.4RC8
--
-- Specialization for an interactive control
--
-- @author  	Manuel Leithner (SFM-Modding)
-- @version 	v2.1
-- @date  		29/08/11
-- @history:	v1.0 - Initial version
--				v2.0 - converted to ls2011
--				v2.1 - improvements
--				v3.0 - converted to work with ls2017
--
--
-- @DATE:        20/02/2017	- v3.0 Converted and adapted to LS/FS 2017 including MP Support by GtX | Andy  @  http://ls-modcompany.com
-- ADDED:                          Support for ls2017 Base Features: Front, High, Workfront & WorkRear lights.
--								   Engine Start/Stop, Warning Flashers, Beacon Lights, CrabSteering State Buttons 1, 2, 3 & 4.
--								   FLT Mode and Folding Parts. Support for Cablight.lua
--
--  
--
-- free for noncommerical-usage
--

InteractiveButtons = {};

function InteractiveButtons.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(InteractiveControl, specializations);
end;

function InteractiveButtons:load(savegame)

	local i=0;
	while true do
		local buttonName = string.format("vehicle.interactiveComponents.buttons.button(%d)", i);	
		if not hasXMLProperty(self.xmlFile, buttonName) then
			break;
		end;
		local name = Utils.getNoNil(g_i18n:getText(getXMLString(self.xmlFile, buttonName .. "#name")), "ERROR");
		local mark = Utils.indexToObject(self.components, getXMLString(self.xmlFile, buttonName .. "#mark"));
		local highlight = getChildAt(mark, 0);
		local size = Utils.getNoNil(getXMLFloat(self.xmlFile, buttonName .. "#size"), 0.1);
		local event = getXMLString(self.xmlFile, buttonName .. "#event");
		local onMessage = g_i18n:getText(Utils.getNoNil(getXMLString(self.xmlFile, buttonName .. "#onMessage"), "ic_button_on"));
		local offMessage =  g_i18n:getText(Utils.getNoNil(getXMLString(self.xmlFile, buttonName .. "#offMessage") , "ic_button_off"));
	
		local button = Button:new(nil, highlight, name, mark, size, event, self, onMessage, offMessage, self.infoBar);
		
		button.synch = Utils.getNoNil(getXMLBool(self.xmlFile, buttonName .. "#synch"), true);

		table.insert(self.interactiveObjects, button);
		i = i + 1;
	end;
end;

function InteractiveButtons:delete()
end;

function InteractiveButtons:mouseEvent(posX, posY, isDown, isUp, button)
end;

function InteractiveButtons:keyEvent(unicode, sym, modifier, isDown)
end;

function InteractiveButtons:update(dt)	
end;

function InteractiveButtons:draw()
end;



--
-- Button Class
-- Specifies an interactive Button
--
-- SFM-Modding
-- @author  Manuel Leithner
-- @date  29/08/11
--
--
-- 2017 v3.0 Feature Added 20/02/2017 by GtX. All credit and ownership to Manuel Leithner. 

Button = {};

function Button:new(node, highlight, name, mark, size, event, vehicle, onMessage, offMessage, infobar)

	local Button_mt = Class(Button, InteractiveComponentInterface);	
    local instance = InteractiveComponentInterface:new(node, highlight, name, mark, size, onMessage, offMessage, infobar, Button_mt);

	instance.vehicle = vehicle;
	instance.event = event;
	
	return instance;	
end;

function Button:delete()
	InteractiveComponentInterface.delete(self);
end;

function Button:mouseEvent(posX, posY, isDown, isUp, button)
	InteractiveComponentInterface.mouseEvent(self, posX, posY, isDown, isUp, button);
end;

function Button:keyEvent(unicode, sym, modifier, isDown)
	InteractiveComponentInterface.keyEvent(self, unicode, sym, modifier, isDown);
end;

function Button:update(dt)
	if self.vehicle ~= nil then
		if self.event == "toggleMotorIgnition" then		
			self.isOpen = self.vehicle.isMotorStarted;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleBeaconLights" then
			self.isOpen = self.vehicle.beaconLightsActive;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleWarningLights" then
			self.isOpen = self.vehicle.turnLightState == Lights.TURNLIGHT_HAZARD;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleCablight" then
			self.isOpen = self.vehicle.cl.turnOn;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleFrontLights" then
			if self.vehicle.numLightTypes >= 1 then
				self.isOpen = (bitAND(self.vehicle.lightsTypesMask, 2^0) == 1 );
			end;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleHighBeamLights" then
			if self.vehicle.numLightTypes >= 4 then
				self.isOpen = (bitAND(self.vehicle.lightsTypesMask, 2^3) == 8 );
			end;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleFrontWorkLights" then
			if self.vehicle.numLightTypes >= 3 then
				self.isOpen = (bitAND(self.vehicle.lightsTypesMask, 2^2) == 4 );
			end;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleRearWorkLights" then
			if self.vehicle.numLightTypes >= 2 then
				self.isOpen = (bitAND(self.vehicle.lightsTypesMask, 2^1) == 2 );
			end;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleFLT" then
			self.isOpen = self.vehicle.crabSteering.state == 1;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleCrabState1" then
			self.isOpen = self.vehicle.crabSteering.state == 1;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleCrabState2" then
			self.isOpen = self.vehicle.crabSteering.state == 2;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleCrabState3" then
			self.isOpen = self.vehicle.crabSteering.state == 3;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleCrabState4" then
			self.isOpen = self.vehicle.crabSteering.state == 4;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleFoldingParts" then
			self.isOpen = self.vehicle:getToggledFoldDirection() > 0;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleFoldingMiddleParts" then
			self.isOpen = self.vehicle:getToggledFoldMiddleDirection() > 0;
		end;
	end;

	InteractiveComponentInterface.update(self, dt);
end;

function Button:draw()
	InteractiveComponentInterface.draw(self);
end;

function Button:doAction(noEventSend, forceAction)
	InteractiveComponentInterface.doAction(self, forceAction);

	if self.vehicle ~= nil then		
		if self.event == "toggleMotorIgnition" then
			if forceAction == nil then
				if not g_currentMission.missionInfo.automaticMotorStartEnabled then
					if self.vehicle.isMotorStarted then
						self.vehicle:stopMotor(true, noEventSend);
					else
						self.vehicle:startMotor(true, noEventSend);
					end;
				end;
			end;
		end;	
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleBeaconLights" then
			if forceAction == nil then
				local state = not self.vehicle.beaconLightsActive;
				self.vehicle:setBeaconLightsVisibility(state, true);
			end;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleWarningLights" then
			if forceAction == nil then
				if self.vehicle.turnLightState == Lights.TURNLIGHT_HAZARD then
					self.vehicle:setTurnLightState(Lights.TURNLIGHT_OFF);
				else
					self.vehicle:setTurnLightState(Lights.TURNLIGHT_HAZARD);
				end;
			end;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleCablight" then
			if forceAction == nil then
				local state = not self.vehicle.cl.turnOn;
				self.vehicle:setCablight(state, true);
			end;
		end;	
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleFrontLights" then
			if self.vehicle.numLightTypes >= 1 then		-------------------
				if forceAction ~= (bitAND(self.vehicle.lightsTypesMask, 2^0) == 1 ) then
					self.vehicle:setLightsTypesMask(bitXOR(self.vehicle.lightsTypesMask, 2^0));
				end;
			end;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleHighBeamLights" then
			if self.vehicle.numLightTypes >= 4 then		-------------------
				if forceAction ~= (bitAND(self.vehicle.lightsTypesMask, 2^3) == 8 ) then
					self.vehicle:setLightsTypesMask(bitXOR(self.vehicle.lightsTypesMask, 2^3));
				end;
			end;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleFrontWorkLights" then
			if self.vehicle.numLightTypes >= 3 then		-------------------
				if forceAction ~= (bitAND(self.vehicle.lightsTypesMask, 2^2) == 4 ) then
					self.vehicle:setLightsTypesMask(bitXOR(self.vehicle.lightsTypesMask, 2^2));
				end;
			end;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleRearWorkLights" then
			if self.vehicle.numLightTypes >= 2 then		-------------------
				if forceAction ~= (bitAND(self.vehicle.lightsTypesMask, 2^1) == 2 ) then
					self.vehicle:setLightsTypesMask(bitXOR(self.vehicle.lightsTypesMask, 2^1));
				end;
			end;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleFLT" then
			if forceAction == nil then
				if self.vehicle.crabSteering.state == 1 then
					self.vehicle:setCrabSteering(2);
				else
					self.vehicle:setCrabSteering(1)
				end;
			end;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleCrabState1" then
			if forceAction == nil then
				self.vehicle:setCrabSteering(1);
			end;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleCrabState2" then
			if forceAction == nil then
				self.vehicle:setCrabSteering(2);
			end;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleCrabState3" then
			if forceAction == nil then
				self.vehicle:setCrabSteering(3);
			end;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleCrabState4" then
			if forceAction == nil then
				self.vehicle:setCrabSteering(4);
			end;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleFoldingParts" then
			if forceAction == nil then
				if self.vehicle:getIsFoldAllowed() then
	                if self.vehicle:getToggledFoldDirection() == self.turnOnFoldDirection then
	                    self.vehicle:setFoldState(self.vehicle:getToggledFoldDirection(), true);
	                else
	                    self.vehicle:setFoldState(self.vehicle:getToggledFoldDirection(), false);
	                end
	            end
			end;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleFoldingMiddleParts" then
			if forceAction == nil then
				local direction = self.vehicle:getToggledFoldMiddleDirection();
				if direction ~= 0 then
					if direction == self.turnOnFoldDirection then
						self.vehicle:setFoldState(direction, false)
					else
						self.vehicle:setFoldState(direction, true)
					end
				end
			end;
		end;
	end;
end;

function Button:onEnter(dt)
	InteractiveComponentInterface.onEnter(self, dt);
end;

function Button:onExit(dt)
	InteractiveComponentInterface.onExit(self, dt);
end;

function Button:setActive()
	InteractiveComponentInterface.setActive(self, isActive);
end;

function Button:setVisible(isVisible)
	InteractiveComponentInterface.setVisible(self, isVisible);
end;